/* -*-c++-*-
 * 
 *    Source     $RCSfile: in_dfap.cc,v $ 
 *    Version    $Revision: 1.1.2.1 $ 
 *    Date       $Date: 2000/02/28 13:58:19 $ 
 *    Author     Christoph Helmberg 
 * 
 --------------------------------------------------------------------------- */

#ident "$Id: in_dfap.cc,v 1.1.2.1 2000/02/28 13:58:19 bzfhelmb Exp $"

#include <fstream.h>
#include <string.h>
#include <ctype.h>
#include <iomanip.h>
#include "sparssym.h"
#include "cmsymspa.h"
#include "cmsingle.h"

//transforms the output of rudy into a maxcut problem readable by
//the bundle algorithm
//if no decrement of the indices is desired the option -nd must be given
//in the command line

int message(const char *s,int i)
{
 cerr<<s<<endl;
 return i;
}

istream& nextnum(istream &in)
{
 char c;
 do {
     if (!in) return in;
     in>>c;
 } while (!isdigit(c));
 in.putback(c);
 return in;
}

istream& skipline(istream &in,int num=1)
{
 char c;
 for(int i=0;i<num;i++){
     do {
         if (!in) return in;
         in.get(c);
     } while (c!='\n');
 }
 return in;
}

   
main(int argc,char **argv)
{
 int do_decrement=1;
 int do_scaling=0;
 int no_inequalities=0;
 int all_inequalities=0;
 const char *usage="\n usage: in_dfap [-i0] [-s] [-ni] [-all]\n   -i0 ... node indices start at 0\n   -s  ... scale constraints\n   -ni ... add no inequality constraints\n   -all... add inequalities on all offdiagonal elements";
 Integer i,j;
 for(i=1;i<argc;i++){
     if (strcmp(argv[i],"-i0")==0){
         do_decrement=0;
         continue;
     }
     if (strcmp(argv[i],"-s")==0){
         do_scaling=1;
         continue;
     }
     if (strcmp(argv[i],"-ni")==0){
         no_inequalities=1;
         all_inequalities=0;
         continue;
     }
     if (strcmp(argv[i],"-all")==0){
         no_inequalities=0;
         all_inequalities=1;
         continue;
     }
     return message(usage,1);
 }
 Integer nodes,edges,parts,dummy;
 cin>>nextnum>>nodes>>nextnum>>edges>>nextnum>>parts;
 //skipline(cin,6);
 Indexmatrix indi(edges,Integer(1),Integer(0));
 Indexmatrix cindi(edges,Integer(1),Integer(0));
 Indexmatrix indj(edges,Integer(1),Integer(0));
 Indexmatrix cindj(edges,Integer(1),Integer(0));
 Matrix val(edges,Integer(1),0.);
 Integer nz=0;
 Integer nc=0;
 for(i=0;i<edges;i++){
     cin>>indi(nz)>>indj(nz)>>val(nz);
     if (do_decrement){
         indi(nz)--;
         indj(nz)--;
     }
     if (val(nz)>=999.999){
         cindi(nc)=indi(nz);
         cindj(nc)=indj(nz);
         nc++;
     }
     else nz++;
 }


 //--- output cost matrix
 Sparsesym L(nodes,nz,indi,indj,val);
 Matrix Ldiag(diag(L));
 L-=sparseDiag(Ldiag);
 Ldiag.init(nodes,1,sum(L)/Real(nodes));
 L*=-1;
 L+=sparseDiag(Ldiag);
 L*=Real(parts-1)/Real(parts)/2.;
 cout<<setprecision(20);
 cout<<nodes<<"\n";
 CMsymsparse cm(L);
 cm.out(cout);

 //--- output total number of constraints
 if (no_inequalities)
     cout<<nodes+nc<<'\n';
 else if (all_inequalities)
     cout<<nodes+ (nodes*(nodes-1))/2<<'\n';
 else
     cout<<nodes+nc+nz<<'\n';
 
 //--- output diagonal constraints
 // xii=1;
 Indexmatrix ind(2,1,Integer(0));
 Real scale=1.;
 val(0)=1.;
 for(i=0;i<nodes;i++){
     ind(0)=i;
     CMsingleton svv(nodes,i,i,1.);
     if (do_scaling) scale=svv.norm();
     svv.multiply(1./scale);
     svv.out(cout);
     cout<<EQU<<" "<<1/scale<<"\n";
 }
 //--- output xij=-1/(k-1) constraints
 // xii+2xij+xjj=2-2/(k-1)
 Symmatrix S;
 if (all_inequalities){
     S.init(nodes,-1.);
 }
 for(i=0;i<nc;i++){
     if (all_inequalities) S(cindi(i),cindj(i))=1.;
     CMsingleton svv(nodes,cindi(i),cindj(i),1.);
     if (do_scaling) scale=svv.norm();
     svv.multiply(1./scale);
     svv.out(cout);
     cout<<EQU<<" "<<(-2./Real(parts-1))/scale<<"\n";
 }

 if (no_inequalities) {
     cout<<endl;
     return 0;
 }

 if (all_inequalities){
     for(i=0;i<nodes;i++){
         for(j=i+1;j<nodes;j++){
             if (S(i,j)>0.) continue;
             CMsingleton svv(nodes,i,j,1.);
             if (do_scaling) scale=svv.norm();
             svv.multiply(1./scale);
             svv.out(cout);
             cout<<GEQ<<" "<<(-2./Real(parts-1))/scale<<"\n";
         }
     }
 }
             
 
 //--- output xij>=-1/(k-1) constraints
 // xii+2xij+xjj>=2-2/(k-1)

 for(i=0;i<nz;i++){
     CMsingleton svv(nodes,indi(i),indj(i),1.);
     if (do_scaling) scale=svv.norm();
     svv.multiply(1./scale);
     svv.out(cout);
     cout<<GEQ<<" "<<(-2./Real(parts-1))/scale<<"\n";
 }
 
 return 0;
}

/* ---------------------------------------------------------------------------
 *    Change log $Log: in_dfap.cc,v $
 *    Change log Revision 1.1.2.1  2000/02/28 13:58:19  bzfhelmb
 *    Change log Reading DIMACS FAP format
 *    Change log
 *    Change log Revision 1.1.1.1  1999/12/15 13:47:48  bzfhelmb
 *    Change log Imported sources
 *    Change log
 *
 *    End of $Source: /kombadon/cvs/cvsroot/bzfhelmb/SBmethod/maxkcut/Attic/in_dfap.cc,v $
 --------------------------------------------------------------------------- */
